@@ -1,13 +1,13 @@
 import os
 import shutil
-from os.path import join, split
+from os.path import join, split, exists
 from pathlib import Path
 from typing import Dict, List, Tuple, Union, Optional
 
 from deep_utils.utils.logging_utils.logging_utils import log_print, value_error_log
 from deep_utils.utils.os_utils.os_path import split_extension
 from deep_utils.utils.shutil_utils.shutil_utils import mv_or_copy
-
+from deep_utils.utils.str_utils.str_utils import StringUtils
 
 def transfer_directory_items(
         in_dir,
@@ -460,6 +460,56 @@ def combine_directory_of_directories(dataset_dir, result_dir, remove_result_dir=
 
 
 class DirUtils:
+    @staticmethod
+    def symbolic_link_move(org_data_list:list[str], des: Union[str, list[str]], remove: bool = False, suffix: str = None,
+                  replace: str | tuple[str, ...] = None, replace_with: str | tuple[str, ...] = None,
+                  current_extension=".nii.gz"):
+
+        if isinstance(des, str) and suffix:
+            os.makedirs(des, exist_ok=True)
+            des = [join(des, DirUtils.split_extension(split(item)[-1],
+                                                      suffix=suffix,
+                                                      current_extension=current_extension)) for
+                   item in org_data_list]
+
+        if replace is not None:
+            des = [StringUtils.replace(item, replace, replace_with) for item in des]
+        if isinstance(des, str):
+            os.makedirs(des, exist_ok=True)
+            for img in org_data_list:
+                target_path = join(des, split(img)[-1])
+                if exists(target_path) and not remove:
+                    raise ValueError(f"target exists: {target_path} for input {img}")
+                elif exists(target_path) and remove:
+                    print(f"Removed target: {target_path}")
+                    os.remove(target_path)
+
+                out = os.system(f"ln -s '{img}' '{target_path}'")
+                if out != 0:
+                    out = os.system(f'ln -s "{img}" "{target_path}"')
+                    if out != 0:
+                        raise ValueError(
+                            f"You messed up bro :) o: {img} and t: {target_path}, check the outputs in the console as well")
+
+
+        elif isinstance(des, list) and len(des) == len(org_data_list):
+            for t, o in zip(des, org_data_list):
+                if exists(t) and not remove:
+                    raise ValueError(f"target exists: {t} for input {o}")
+                elif exists(t) and remove:
+                    print(f"Removed target: {t}")
+                    os.remove(t)
+                out = os.system(f"ln -s '{o}' '{t}'")
+                if out != 0:
+                    out = os.system(f'ln -s "{o}" "{t}"')
+                    if out != 0:
+                        raise ValueError(
+                            f"You messed up bro :) o: {o} and t: {t}, check the outputs in the console as well")
+        else:
+            raise "Dude what have you done:)"
+
+
+
     @staticmethod
     def split_dir_of_dir(
             in_dir,
@@ -646,7 +696,7 @@ class DirUtils:
                            dir_depth: int = -1,
                            exact_depth: bool = False,
                            return_dict: bool = False,
-                           ends_with:Optional[Union[str, List[str]]] = None,
+                           ends_with: Optional[Union[str, List[str]]] = None,
                            ) -> Union[List[str], Dict[str, str]]:
         """
         Returns the full path objects in a directory
@@ -889,6 +939,7 @@ class DirUtils:
             return True
         else:
             raise ValueError(f"ext: {ext} is not supported!")
+
     @staticmethod
     def file_incremental(file_path: str | None, artifact_type="prefix", artifact_value=0, extra_punctuation="_",
                          add_artifact_value=False, dir_items: list[str] | None = None):
@@ -978,12 +1029,16 @@ class DirUtils:
         else:
             return False
 
-    def split(path: str, depth: int = 1, continuous: bool = False, list_it: bool = False):
+    def split(path: str, depth: int = 1, continuous: bool = False, list_it: bool = False,
+              join_del: str = None, return_right: bool = True):
         """
 
         :param path:
         :param depth:
         :param continuous:
+        :param list_it:
+        :param join_del:
+        :param return_right:
         :return:
         >>> DirUtils.split("/pooya/ali/saeed/wow.txt", 3)
         'ali'
@@ -997,8 +1052,12 @@ class DirUtils:
         'ali/saeed'
         >>> DirUtils.split("/pooya/ali/saeed", 2, continuous=True, list_it=True)
         ['ali', 'saeed']
+        >>> DirUtils.split("/pooya/ali/saeed", 2, continuous=True, join_del=",")
+        'ali,saeed'
         >>> DirUtils.split("/pooya/ali/saeed", 0)
         ['pooya', 'ali', 'saeed']
+        >>> DirUtils.split("/pooya/ali/saeed", depth=1, return_right=False)
+        '/pooya/ali'
         """
         if depth == 0:
             outputs = []
@@ -1009,9 +1068,15 @@ class DirUtils:
                 outputs.insert(0, p)
             return outputs
         elif depth == 1:
-            import warnings
-            warnings.warn("Use os.path.split(path)[-1] for depth=1 :)")
-            return split(path)[-1]
+            if return_right:
+                import warnings
+                warnings.warn("Use os.path.split(path)[-1] for depth=1 :)")
+
+                return split(path)[-1]
+            else:
+                import warnings
+                warnings.warn("Use os.path.split(path)[0] for depth=1 and return_right=False :)")
+                return split(path)[0]
         elif depth < 1:
             raise ValueError("depth should not be lower than 1")
         else:
@@ -1019,8 +1084,10 @@ class DirUtils:
                 p = path
                 for _ in range(depth - 1):
                     p = split(p)[0]
-
-                output = split(p)[-1]
+                if return_right:
+                    output = split(p)[-1]
+                else:
+                    output = split(p)[0]
             else:
                 outputs = []
                 for _ in range(depth):
@@ -1029,13 +1096,221 @@ class DirUtils:
                 if list_it:
                     output = outputs
                 else:
-                    output = os.path.join(*outputs)
+                    if join_del is None:
+                        output = os.path.join(*outputs)
+                    else:
+                        output = join_del.join(item for item in outputs)
             return output
 
     @staticmethod
-    def get_dir_time(directory: str):
-        return {item.split(" ")[-1]: " ".join( item.split(" ")[-4: -1]) for item in DirUtils.execute_command(f"ls -l {directory}").split("\n") if item}
-mkdir_incremental = DirUtils.mkdir_incremental
+    def get_file_time(filepath: str):
+        item = DirUtils.execute_command(f"ls -l '{filepath}'")
+        return " ".join(item.split(" ")[-4: -1])
 
-if __name__ == '__main__':
-    print() #
+    @staticmethod
+    def get_dir_time(directory: str):
+        return {item.split(" ")[-1]: " ".join(item.split(" ")[-4: -1]) for item in
+                DirUtils.execute_command(f"ls -l '{directory}'").split("\n") if item}
+
+    @staticmethod
+    def open_dir(directory: str):
+        os.system(f"nautilus '{directory}'")
+
+    @staticmethod
+    def get_symbolink(directory: str) -> Dict[str, str]:
+        output = [item.split("->") for item in DirUtils.execute_command(f"ls -l {directory}").split("\n")[1:]]
+        output = {" ".join(item[0].strip().split(" ")[9:]).strip(): item[1].strip() for item in output if
+                  len(item) == 2}
+        return output
+
+    @staticmethod
+    def open_image(path):
+        import sys
+        import subprocess
+        imageViewerFromCommandLine = {'linux': 'xdg-open',
+                                      'win32': 'explorer',
+                                      'darwin': 'open'}[sys.platform]
+        subprocess.run([imageViewerFromCommandLine, path])
+
+    @staticmethod
+    def remove_empty_dirs(directory: str, verbose: bool = True):
+        from tqdm import tqdm
+        if os.path.exists(directory):
+            all_dirs = os.walk(directory)
+            pbar = tqdm(all_dirs, total=len(all_dirs))
+            for root, dirs, filenames in pbar:
+                for dir_ in dirs:
+                    try:
+                        os.remove(dir_)
+                    except:
+                        if verbose:
+                            print(f"[Warning] {dir_} is not empty")
+                pbar.update()
+
+    @staticmethod
+    def is_empty(directory: str):
+        if exists(directory):
+            for root, dirs, filenames in os.walk(directory):
+                for _ in dirs:
+                    return False
+                for _ in filenames:
+                    return False
+        return True
+
+    @staticmethod
+    def safe_item_move(base_directory: str, target_directory: str, remove_if_sizes_are_the_same: bool = True,
+                       keep_the_largest_if_sizes_are_not_the_same: bool = True,
+                       remove_base_empty_dir: bool = True, verbose: bool = True,
+                       copy: bool = False, endswith: str | tuple[str] = None,
+                       not_endswith: str | tuple[str] = None,
+                       min_size: int = None):
+        """
+        This only works for items not directories
+        :param base_directory:
+        :param target_directory:
+        :param remove_base_empty_dir:
+        :param verbose:
+        :param remove_if_sizes_are_the_same:
+        :param min_size: this is based on megabyte
+        :return:
+        """
+        from tqdm import tqdm
+        target_directory_created = False
+        if not exists(target_directory):
+            os.makedirs(target_directory, exist_ok=True)
+            target_directory_created = True
+        all_items = list(os.listdir(base_directory))
+        for base_item_name in tqdm(all_items, total=len(all_items), desc=f"Items: {base_directory} --> {target_directory}", disable=not verbose):
+            if endswith and not base_item_name.endswith(endswith):
+                continue
+            if not_endswith and base_item_name.endswith(not_endswith):
+                continue
+            target_item_path = join(target_directory, base_item_name)
+            base_item_path = join(base_directory, base_item_name)
+
+            if min_size: # skip if size is less than specified!
+                base_size = os.path.getsize(base_item_path) / 1024 / 1024 # MB
+                if base_size < min_size:
+                    continue
+
+            if not exists(target_item_path):
+                mv_or_copy(base_item_path, target_item_path, mode="cp" if copy else "mv")
+            else:
+                # check the sizes
+                base_size = os.path.getsize(base_item_path)
+                target_size = os.path.getsize(target_item_path)
+                if target_size == base_size and remove_if_sizes_are_the_same:
+                    os.remove(base_item_path)
+                elif target_size >= base_size and keep_the_largest_if_sizes_are_not_the_same:
+                    os.remove(base_item_path)
+                elif target_size < base_size and keep_the_largest_if_sizes_are_not_the_same:
+                    mv_or_copy(base_item_path, target_item_path, mode="cp" if copy else "mv")
+                else:
+                    if verbose:
+                        print(f"[WARNING] {base_item_path} with size: {base_size} is different from {target_item_path} with size: {target_size}")
+        output = True
+        if DirUtils.is_empty(base_directory) and remove_base_empty_dir:
+            os.rmdir(base_directory)
+        elif not remove_base_empty_dir:
+            pass
+        else:
+            if verbose:
+                print(f"There are some items/directories left in {base_directory}, preventing the base removal")
+            output = False
+        if target_directory_created and len(os.listdir(target_directory)) == 0:
+            os.rmdir(target_directory)
+        return output
+
+    @staticmethod
+    def move_dir_of_dirs(base_dir: str, target_dir: str, remove_empty_base_dirs:bool=True, verbose: bool = True, endswith: str | tuple[str] = None, move_n_samples: int = None, n_jobs: int = 1):
+        """
+
+        :param base_dir:
+        :param target_dir:
+        :param remove_empty_base_dirs:
+        :param verbose:
+        :param endswith:
+        :param move_n_samples: This is only applied to the main directory not the inner ones.
+        :return:
+        """
+        from tqdm import tqdm
+
+        def shutil_move(base_directory_, target_directory_, endswith_):
+            if endswith_:
+                if base_directory_.endswith(endswith_):
+                    shutil.move(base_directory_, target_directory_)
+            else:
+                shutil.move(base_directory_, target_directory_)
+
+        def _move(base_directory_, target_dir_, endswith_):
+            if not DirUtils.is_empty(base_directory_):
+                target_directory_ = join(target_dir_, split(base_directory_)[-1])
+                if exists(target_directory_):
+                    if DirUtils.is_empty(target_directory_):
+                        os.rmdir(target_directory_)
+                        shutil_move(base_directory_, target_directory_, endswith_)
+                    else:
+                        DirUtils.move_dir_of_dirs(base_directory_,
+                                                  target_directory_,
+                                                  remove_empty_base_dirs=remove_empty_base_dirs,
+                                                  endswith=endswith,
+                                                  verbose=verbose)
+                else:
+                    os.makedirs(split(target_directory_)[0], exist_ok=True)
+                    shutil_move(base_directory_, target_directory_, endswith_)
+            else:
+                if remove_empty_base_dirs:
+                    os.rmdir(base_directory_)
+
+        all_base_dirs = DirUtils.list_dir_full_path(base_dir, only_directories=True)[:move_n_samples]
+        if len(all_base_dirs):
+            if n_jobs > 1:
+                from joblib import Parallel, delayed
+                list(tqdm(Parallel(return_as="generator", n_jobs=n_jobs)(delayed(_move)(base_directory, target_dir, endswith) for base_directory in all_base_dirs), total=len(all_base_dirs), desc=f"Dirs: {base_dir} --> {target_dir}"))
+            else:
+                for base_directory in tqdm(all_base_dirs, total=len(all_base_dirs), desc=f"Dirs: {base_dir} --> {target_dir}"):
+                    _move(base_directory, target_dir, endswith)
+
+            if DirUtils.is_empty(base_dir)and remove_empty_base_dirs:
+                os.rmdir(base_dir)
+            else:
+                if verbose:
+                    print(f"[INFO] There are some files(not directories) in {base_dir} preventing it from being removed!")
+        else:
+            if endswith and not base_dir.endswith(endswith):
+                return
+            fixed = DirUtils.safe_item_move(base_dir, target_dir, remove_base_empty_dir=remove_empty_base_dirs, verbose=verbose)
+            if verbose and not fixed:
+                print(f"Directory: {base_dir} and {target_dir} contain same data")
+    @staticmethod
+    def list_items_walk(directory_path: str, endswith: Union[str, tuple[str, ...]]=None):
+        output = []
+        for root, _, filenames in os.walk(directory_path):
+            for filename in filenames:
+                if filename.endswith(endswith):
+                    output.append(join(root, filename))
+
+
+
+    @staticmethod
+    def list_items_scandir(directory_path: str, endswith: Union[str, tuple[str, ...]] = None,
+                           not_endswith: Union[str, tuple[str, ...]] = None):
+        if endswith is not None and not_endswith is None:
+            for entry in os.scandir(directory_path):
+                if not entry.name.startswith('.'):
+                    if entry.is_file():
+                        if entry.name.endswith(endswith):
+                            yield entry.path
+                    else:
+                        yield from DirUtils.list_items_scandir(entry.path, endswith=endswith)
+
+        elif endswith is not None and not_endswith is not None:
+            for entry in os.scandir(directory_path):
+                if not entry.name.startswith('.'):
+                    if entry.is_file():
+                        if entry.name.endswith(endswith) and not entry.name.endswith(not_endswith):
+                            yield entry.path
+                    else:
+                        yield from DirUtils.list_items_scandir(entry.path, endswith=endswith, not_endswith=not_endswith)
+
+mkdir_incremental = DirUtils.mkdir_incremental
